jest.mock('./../lib/paths.js');
jest.mock('./../lib/logger.js');
jest.mock('./../lib/dependency.js');
jest.mock('./../lib/flowTyped.js');
jest.mock('./../lib/exec.js');
console.error = jest.fn();

const path = require('./../lib/paths.js');

const dependency = require('./../lib/dependency.js');

const flowTyped = require('./../lib/flowTyped.js');

const logger = require('./../lib/logger.js');

const exec = require('./../lib/exec.js');

const installFlowTypes = require('./install-types.js');

describe('install-types', () => {
  afterEach(() => {
    // $FlowFixMe: Ignore errors since the jest type-def is out of date.
    jest.restoreAllMocks();
    jest.clearAllMocks();
  });
  it('should export an function', () => {
    expect(typeof installFlowTypes).toBe('function');
  });
  it('should update the flow-typed cache and afterwards install types in all packages', async () => {
    path.resolveMonoRepoRootPath.mockReturnValue('/foo');
    path.resolveMonoRepoPackagePaths.mockReturnValue(['/foo/bar', '/foo/baz']);
    flowTyped.parseArgs.mockReturnValue(['--overwrite']);
    await installFlowTypes();
    expect(exec.asyncWithRetries.mock.calls).toMatchSnapshot();
  });
  it('should fatally exit the process if something went wrong during the installation', async () => {
    path.resolveMonoRepoRootPath.mockReturnValue('/foo');
    path.resolveMonoRepoPackagePaths.mockReturnValue(['/foo/bar', '/foo/baz']);
    flowTyped.parseArgs.mockReturnValue(['--overwrite']);
    dependency.readPackageJson.mockReturnValue({
      name: 'myFooPackageName'
    });
    exec.asyncWithRetries.mockReturnValueOnce(null).mockReturnValueOnce(Promise.reject(new Error('Foo')));
    await installFlowTypes();
    expect(logger.error.mock.calls).toMatchSnapshot();
    expect(console.error).toHaveBeenCalled();
    expect(console.error.mock.calls[0][0].message).toBe('Foo');
  });
});